 
/********************************************************/
/* Chargement fichier off                               */
/********************************************************/

/* inclusion des fichiers d'en-tete Glut */
#include <math.h>
#include <stdlib.h>
#include <iostream>
#include <GL/glut.h>
#include <vector>

/* Notre structure point */
typedef struct
{
  double x[3];
} sommet;
typedef struct
{
  int index[3];
} index_triangle;
typedef struct
{
  std::vector <sommet> geometry;
  std::vector <index_triangle> connectivity;

} mesh;


mesh maillage;

char presse,presse_d=0;
int anglex,angley,x,y,xold,yold;
double zoom=-5.0;

/* Prototype des fonctions */
void affichage();
void clavier(unsigned char touche,int x,int y);
void reshape(int x,int y);
void idle();
void mouse(int bouton,int etat,int x,int y);
void mousemotion(int x,int y);

void load_off_file(const char* filename);


int main(int argc,char **argv)
{
  /* initialisation de glut et creation
     de la fenetre */
  glutInit(&argc,argv);
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  glutInitWindowPosition(0,0);
  glutInitWindowSize(550,550);
  glutCreateWindow("Test");

  /* Initialisation d'OpenGL */
  glClearColor(0.0,0.0,0.0,0.0);
  glColor3f(1.0,1.0,1.0);
  glPointSize(2.0);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_COLOR_MATERIAL);

  /* enregistrement des fonctions de rappel */
  glutDisplayFunc(affichage);
  glutKeyboardFunc(clavier);
  glutReshapeFunc(reshape);
  glutMouseFunc(mouse);
  glutMotionFunc(mousemotion);


  //light
  float light_ambient[4]={0.4,0.4,0.4,1.0};
  float light_diffuse[4]={0.4,0.4,0.4,1.0};
  float light_specular[4]={0.3,0.3,0.3,1.0};
  float light_position [4] = {2.0,100.0,100.0,1.0};

  float material_ambient [4] = {1.0,1.0,1.0,1.0};
  float material_diffuse [4] = {1.0,1.0,1.0,1.0};
  float material_specular [4] = {1.0,1.0,1.0,1.0};
  float material_exponent [1] = {60.0};


  glShadeModel(GL_SMOOTH);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material_ambient);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material_diffuse);  
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, material_specular);  
  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, material_exponent);
  glEnable(GL_DEPTH_TEST);




  /* Charge fichier off */
  load_off_file("venus.off");

  /* Entree dans la boucle principale glut */
  glutMainLoop();
  return 0;
}

void load_off_file(const char* filename)
{

#define SIZE_BUFFER 2048
  char buffer[SIZE_BUFFER]={'\0'};
  
  
  FILE *fid=NULL;
  fid=fopen(filename,"r");
  if(fid==NULL)
    {printf("ERROR loading %s in load_off_file\n",filename);exit(-1);}


  while(strcmp(buffer,"OFF")!=0)
    {
      if(fscanf(fid,"%s",buffer)==EOF)
	{printf("ERROR file %s is not off file\n",filename);exit(-1);}
      if(buffer[0]=='#')//comments
	fgets(buffer,sizeof(buffer),fid);
    }


  //pass comments
  fscanf(fid,"%s",buffer);
  while(buffer[0]=='#')//comments
    {
      fgets(buffer,sizeof(buffer),fid);
      fscanf(fid,"%s",buffer);
    }


  //read Number of vertices
  int N_vertex=0,N_polygon=0;
  int k_vertex=0;
  N_vertex=atoi(buffer);//N_vertex
  fscanf(fid,"%d",&N_polygon);//N_polygon
  fgets(buffer,sizeof(buffer),fid);//read line


  //read vertices
  float X[3];
  sommet temp_sommet;
  while(k_vertex<N_vertex)
    {
      if(fscanf(fid,"%f %f %f",X,X+1,X+2)!=3)
	{
	  fscanf(fid,"%s",buffer);
	  if(buffer[0]!='#')
	    {printf("ERROR loading %s\n",filename);exit(-1);}
	  else//read line
	    fgets(buffer,sizeof(buffer),fid);
	}
      else
	{
	  for(int k=0;k<3;k++)
	    temp_sommet.x[k]=X[k];
	  maillage.geometry.push_back(temp_sommet);

	  k_vertex++;
	}
    }

  //read connectivity
  int size_poly=0;
  int k_polygon=0;
  int k_current_polygon=0;
  int temp=0;
  std::vector <int> v_poly;
  index_triangle temp_triangle;
  for(k_polygon=0;k_polygon<N_polygon;k_polygon++)
    {
      if(fscanf(fid,"%d",&size_poly)!=1)
	{
	  fscanf(fid,"%s",buffer);
	  if(buffer[0]!='#')
	    {printf("ERROR loading triangles in load_off_file %s\n",filename);exit(-1);}
	  else//read line
	    {
	      fgets(buffer,sizeof(buffer),fid);
	      k_polygon--;
	    }
	}
      else
	{
	  v_poly.resize(0);
	  for(k_current_polygon=0;k_current_polygon<size_poly;k_current_polygon++)
	    {
	      fscanf(fid,"%d",&temp);
	      v_poly.push_back(temp);
	    }

	  for(unsigned int k=0;k<v_poly.size()-2;k++)
	    {
	      temp_triangle.index[0]=v_poly[0];
	      temp_triangle.index[1]=v_poly[k+1];
	      temp_triangle.index[2]=v_poly[k+2];
	      maillage.connectivity.push_back(temp_triangle);
	    }


	}
    }

  printf("[Chargement fichier %s OK]\n",filename);

}


void affichage()
{
  int i;


  /* effacement de l'image avec la couleur de fond */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


  //Projection
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity ();
  gluPerspective (45, 1.0, 0.01,100.0);

  
  //Modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(0.0,0.0,zoom);
  glRotatef(-angley,1.0,0.0,0.0);
  glRotatef(-anglex,0.0,1.0,0.0);

  int N=maillage.connectivity.size();
  glBegin(GL_TRIANGLES);

  double X0[3],X1[3],X2[3];
  double u[3],v[3];
  double normal[3],n;
  for(i=0;i<N;i++)
    {
      for(int k=0;k<3;k++)
	{
	  X0[k] = maillage.geometry[maillage.connectivity[i].index[0]].x[k];
	  X1[k] = maillage.geometry[maillage.connectivity[i].index[1]].x[k];
	  X2[k] = maillage.geometry[maillage.connectivity[i].index[2]].x[k];
	}
      for(int k=0;k<3;u[k]=X1[k]-X0[k],v[k]=X2[k]-X0[k],k++);
      
      //color
      glColor3d(0.8,0.8,0.8);

      //normal
      normal[0]=u[1]*v[2]-u[2]*v[1];
      normal[1]=u[2]*v[0]-u[0]*v[2];
      normal[2]=u[0]*v[1]-u[1]*v[0];
      n = powf(normal[0]*normal[0]+normal[1]*normal[1]+normal[2]*normal[2],0.5);
      if(n>=0.0001)
	for(int k=0;k<3;k++)
	  normal[k]/=n;
      glNormal3dv(normal);

      //vertex
      glVertex3dv(X0);
      glVertex3dv(X1);
      glVertex3dv(X2);

    }

  glEnd();

  
  /* On echange les buffers */
  glutSwapBuffers();
}

void clavier(unsigned char touche,int x,int y)
{
  x=0;y=0;
  switch (touche)
    {
    case 'p': /* affichage du carre plein */
      glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
      glutPostRedisplay();
      break;
    case 'f': /* affichage en mode fil de fer */
      glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
      glutPostRedisplay();
      break;
    case 's' : /* Affichage en mode sommets seuls */
      glPolygonMode(GL_FRONT_AND_BACK,GL_POINT);
      glutPostRedisplay();
      break;
    case 'd':
      glEnable(GL_DEPTH_TEST);
      glutPostRedisplay();
      break;
    case 'D':
      glDisable(GL_DEPTH_TEST);
      glutPostRedisplay();
      break;
    case 'q' : /*la touche 'q' permet de quitter le programme */
      exit(0);
    }
}

void reshape(int x,int y)
{
  if (x<y)
    glViewport(0,(y-x)/2,x,x);
  else 
    glViewport((x-y)/2,0,y,y);
}

void mouse(int button, int state,int x,int y)
{
  /* si on appuie sur le bouton gauche */
  if(state == GLUT_DOWN) 
  {
    xold = x; /* on sauvegarde la position de la souris */
    yold=y;
  }
  if(button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
    presse=1;
  if(button == GLUT_LEFT_BUTTON && state==GLUT_UP)
    presse=0;

  if(button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
    presse_d=1;
  if(button == GLUT_RIGHT_BUTTON && state == GLUT_UP)
    presse_d=0;

}

void mousemotion(int x,int y)
{
  if (presse) /* si le bouton gauche est presse */
    {
      /* on modifie les angles de rotation de l'objet
	 en fonction de la position actuelle de la souris et de la derniere
	 position sauvegardee */
      anglex=anglex+(x-xold); 
      angley=angley+(y-yold);
      glutPostRedisplay(); /* on demande un rafraichissement de l'affichage */
    }
  if(presse_d)
    {
      zoom = zoom + (y-yold)/50.0;
      glutPostRedisplay(); /* on demande un rafraichissement de l'affichage */
    }

  
  xold=x; /* sauvegarde des valeurs courante de le position de la souris */
  yold=y;
}

 


	
	
